/*
 * folder.cpp
 *
 */

// Standard library imports
#include <stdexcept>
#include <string>
#include <dirent.h>

// Third party imports
#include <opencv2/highgui/highgui.hpp>

// Local imports
#include "folder.h"

// Namespaces
using namespace std;

/*!********************************************************************************
 * Load images from a directory
 **********************************************************************************/

void loadFolder(const string & dirpath, vector<Mat> & img) {
	DIR *dir;
	struct dirent *ent;
	vector<string> names;
	vector<string>::iterator it;

	// Initialization
	img.clear();
	if (not (dir = opendir(dirpath.c_str())))
		throw runtime_error("Could not open directory: " + dirpath);

	// Read all the files within directory (except hidden files and ./)
	while (ent = readdir(dir))
		if (ent->d_name[0] != '.')
			names.push_back(ent->d_name);

	// Load the files in order
	sort(names.begin(), names.end());
	for (it = names.begin(); it != names.end(); ++it) {
		img.push_back(imread(dirpath + *it, CV_LOAD_IMAGE_COLOR));
		if (img.back().empty())
			throw runtime_error("Could not read one of the images: " + *it);
	}

	// Close directory
	if (closedir(dir))
		throw runtime_error("Could not close directory: " + dirpath);
}

