/* source : http://www.niac.man.ac.uk/Tina/lxr/http/source/math/numeric/svd.c */
/* TINA Cross Reference */

#include "svd.h"

/* smc 2/2/93 */
#ifdef Print
#undef Print
#endif
/* smc 2/2/93 */

#define Print(x)  { if(out_text!=NULL) out_text(x); }

/* This macro is now redundant as function exists but kept incase
 * function is bad re-implementation of macro */
#define Radius(u,v)          ( (Au=Abs((u))) > (Av=Abs((v))) ? \
                             (Aw=Av/Au, Au*sqrt(1.+Aw*Aw)) : \
                             (Av ? (Aw=Au/Av, Av*sqrt(1.+Aw*Aw)) : 0.0) )
#define Sign(u,v)               ( (v)>=0.0 ? Abs(u) : -Abs(u) )

#define Max(u,v)                ( (u)>(v)?  (u) : (v) )

#define Abs(x)                  ( (x)>0.0?  (x) : (-(x)) )

static void (*out_text)() = NULL;

static double radius(double u, double v) {
	double Au, Av, Aw;

	Au = Abs(u);
	Av = Abs(v);

	if (Au > Av) {
		Aw = Av / Au;
		return (Au * sqrt(1. + Aw * Aw));
	} else {
		if (Av) {
			Aw = Au / Av;
			return (Av * sqrt(1. + Aw * Aw));
		} else
			return 0.0;
	}
}

void svd_err_func(void(*text)( /* ??? */)) {
	out_text = text;
}

/*************************** SVDcmp *****************************************
 * Given matrix A[m][n], m>=n, using svd decomposition A = U W V' to get     *
 * U[m][n], W[n][n] and V[n][n], where U occupies the position of A.         *
 * NOTE: if m<n, A should be filled up to square with zero rows.             *
 *       A[m][n] has been destroyed by U[m][n] after the decomposition.      *
 ****************************************************************************/
void SVDcmp(double *a, int m, int n, double *w, double *v) {
	/* BUG `nm' may be used uninitialized in this function */
	int flag, i, its, j, jj, k, l, nm, nm1 = n - 1, mm1 = m - 1;
	double c, f, h, s, x, y, z;
	double anorm = 0.0, g = 0.0, scale = 0.0;
	double *rv1;

	if (m < n)
		fprintf(stderr, "Error in SVD function: m must be larger or equal to n\n");

	rv1 = (double *) malloc((unsigned) n * sizeof(double));

	/* Householder reduction to bidigonal form */
	for (i = 0; i < n; i++) {
		l = i + 1;
		rv1[i] = scale * g;
		g = s = scale = 0.0;
		if (i < m) {
			for (k = i; k < m; k++)
				scale += Abs(a[k*n+i]);
			if (scale) {
				for (k = i; k < m; k++) {
					a[k * n + i] /= scale;
					s += a[k * n + i] * a[k * n + i];
				}
				f = a[i * n + i];
				g = -Sign(sqrt(s), f);
				h = f * g - s;
				a[i * n + i] = f - g;
				if (i != nm1) {
					for (j = l; j < n; j++) {
						for (s = 0.0, k = i; k < m; k++)
							s += a[k * n + i] * a[k * n + j];
						f = s / h;
						for (k = i; k < m; k++)
							a[k * n + j] += f * a[k * n + i];
					}
				}
				for (k = i; k < m; k++)
					a[k * n + i] *= scale;
			}
		}
		w[i] = scale * g;
		g = s = scale = 0.0;
		if (i < m && i != nm1) {
			for (k = l; k < n; k++)
				scale += Abs(a[i*n+k]);
			if (scale) {
				for (k = l; k < n; k++) {
					a[i * n + k] /= scale;
					s += a[i * n + k] * a[i * n + k];
				}
				f = a[i * n + l];
				g = -Sign(sqrt(s), f);
				h = f * g - s;
				a[i * n + l] = f - g;
				for (k = l; k < n; k++)
					rv1[k] = a[i * n + k] / h;
				if (i != mm1) {
					for (j = l; j < m; j++) {
						for (s = 0.0, k = l; k < n; k++)
							s += a[j * n + k] * a[i * n + k];
						for (k = l; k < n; k++)
							a[j * n + k] += s * rv1[k];
					}
				}
				for (k = l; k < n; k++)
					a[i * n + k] *= scale;
			}
		}
		anorm = Max(anorm, (Abs(w[i]) + Abs(rv1[i])));
	}

	/* Accumulation of right-hand transformations */
	for (i = n - 1; i >= 0; i--) {
		if (i < nm1) {
			if (g) {
				/* double division to avoid possible underflow */
				for (j = l; j < n; j++)
					v[j * n + i] = (a[i * n + j] / a[i * n + l]) / g;
				for (j = l; j < n; j++) {
					for (s = 0.0, k = l; k < n; k++)
						s += a[i * n + k] * v[k * n + j];
					for (k = l; k < n; k++)
						v[k * n + j] += s * v[k * n + i];
				}
			}
			for (j = l; j < n; j++)
				v[i * n + j] = v[j * n + i] = 0.0;
		}
		v[i * n + i] = 1.0;
		g = rv1[i];
		l = i;
	}
	/* Accumulation of left-hand transformations */
	for (i = n - 1; i >= 0; i--) {
		l = i + 1;
		g = w[i];
		if (i < nm1)
			for (j = l; j < n; j++)
				a[i * n + j] = 0.0;
		if (g) {
			g = 1.0 / g;
			if (i != nm1) {
				for (j = l; j < n; j++) {
					for (s = 0.0, k = l; k < m; k++)
						s += a[k * n + i] * a[k * n + j];
					f = (s / a[i * n + i]) * g;
					for (k = i; k < m; k++)
						a[k * n + j] += f * a[k * n + i];
				}
			}
			for (j = i; j < m; j++)
				a[j * n + i] *= g;
		} else
			for (j = i; j < m; j++)
				a[j * n + i] = 0.0;
		++a[i * n + i];
	}
	/* diagonalization of the bidigonal form */
	for (k = n - 1; k >= 0; k--) { /* loop over singlar values */
		for (its = 0; its < 30; its++) { /* loop over allowed iterations */
			flag = 1;
			for (l = k; l >= 0; l--) { /* test for splitting */
				nm = l - 1; /* note that rv1[l] is always zero */
				if (Abs(rv1[l]) + anorm == anorm) {
					flag = 0;
					break;
				}
				if (Abs(w[nm]) + anorm == anorm)
					break;
			}
			if (flag) {
				c = 0.0; /* cancellation of rv1[l], if l>1 */
				s = 1.0;
				for (i = l; i <= k; i++) {
					f = s * rv1[i];
					if (Abs(f) + anorm != anorm) {
						g = w[i];
						h = radius(f, g);
						w[i] = h;
						h = 1.0 / h;
						c = g * h;
						s = (-f * h);
						for (j = 0; j < m; j++) {
							y = a[j * n + nm];
							z = a[j * n + i];
							a[j * n + nm] = y * c + z * s;
							a[j * n + i] = z * c - y * s;
						}
					}
				}
			}
			z = w[k];
			if (l == k) { /* convergence */
				if (z < 0.0) {
					w[k] = -z;
					for (j = 0; j < n; j++)
						v[j * n + k] = (-v[j * n + k]);
				}
				break;
			}
			if (its == 30)
				Print("No convergence in 30 SVDCMP iterations");
			x = w[l]; /* shift from bottom 2-by-2 minor */
			nm = k - 1;
			y = w[nm];
			g = rv1[nm];
			h = rv1[k];
			f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0 * h * y);
			g = radius(f, 1.0);
			/* next QR transformation */
			f = ((x - z) * (x + z) + h * ((y / (f + Sign(g, f))) - h)) / x;
			c = s = 1.0;
			for (j = l; j <= nm; j++) {
				i = j + 1;
				g = rv1[i];
				y = w[i];
				h = s * g;
				g = c * g;
				z = radius(f, h);
				rv1[j] = z;
				c = f / z;
				s = h / z;
				f = x * c + g * s;
				g = g * c - x * s;
				h = y * s;
				y = y * c;
				for (jj = 0; jj < n; jj++) {
					x = v[jj * n + j];
					z = v[jj * n + i];
					v[jj * n + j] = x * c + z * s;
					v[jj * n + i] = z * c - x * s;
				}
				z = radius(f, h);
				w[j] = z; /* rotation can be arbitrary id z=0 */
				if (z) {
					z = 1.0 / z;
					c = f * z;
					s = h * z;
				}
				f = (c * g) + (s * y);
				x = (c * y) - (s * g);
				for (jj = 0; jj < m; jj++) {
					y = a[jj * n + j];
					z = a[jj * n + i];
					a[jj * n + j] = y * c + z * s;
					a[jj * n + i] = z * c - y * s;
				}
			}
			rv1[l] = 0.0;
			rv1[k] = f;
			w[k] = x;
		}
	}
	free((void *) rv1);
}

/******************************* SVDbksb ****************************
 * Given A[m][n], b[m], solves A x = b in the svd form U W V'x = b   *
 * so x = V U'b/W                                                    *
 * No input quantities are destroyed, so sequential calling is OK    *
 ********************************************************************/
void SVDb(double *u, double *w, double *v, int m, int n, double *b, double *x) {
	int jj, j, i;
	double s, *tmp;

	tmp = (double *) malloc((unsigned) n * sizeof(double));
	for (j = 0; j < n; j++) { /* calculate <U,b> */
		s = 0.0;
		if (w[j]) { /* nonzero result only if nonzero w[j] */
			for (i = 0; i < m; i++)
				s += u[i * n + j] * b[i];
			s /= w[j]; /* premultiplied by inverse W */
		}
		tmp[j] = s;
	}
	for (j = 0; j < n; j++) { /* V(U'b/W) */
		s = 0.0;
		for (jj = 0; jj < n; jj++)
			s += v[j * n + jj] * tmp[jj];
		x[j] = s;
	}
	free((void *) tmp);
}

