/*
 * rotations.cpp
 *
 */

// Standard library imports
#include <iostream>

// Third party imports
#include <opencv2/core/core.hpp>

// Local imports
#include "rotations.h"

// Namespaces
using namespace cv;
using namespace std;

Mat rotationFromAxis(const Vec3d & axis, double theta) {
	// Rotation matrix for a rotation of theta around the vector 'axis'
	double x, y, z, c, s, C, xs, ys, zs, xC, yC, zC, xyC, yzC, zxC;
	Mat_<double> m(3, 3);

	x = axis[0];
	y = axis[1];
	z = axis[2];
	c = cos(theta);
	s = sin(theta);
	C = 1 - c;
	xs = x * s;
	ys = y * s;
	zs = z * s;
	xC = x * C;
	yC = y * C;
	zC = z * C;
	xyC = x * yC;
	yzC = y * zC;
	zxC = z * xC;

	m(0, 0) = x * xC + c;
	m(0, 1) = xyC - zs;
	m(0, 2) = zxC + ys;
	m(1, 0) = xyC + zs;
	m(1, 1) = y * yC + c;
	m(1, 2) = yzC - xs;
	m(2, 0) = zxC - ys;
	m(2, 1) = yzC + xs;
	m(2, 2) = z * zC + c;

	return m;
}

Mat rotationFromVectors(const Vec3d & u, const Vec3d & v) {
	// Rotation matrix that maps u → v, assuming {u,v} are unit vectors
	double x, y, z, c, s, C, xs, ys, zs, xC, yC, zC, xyC, yzC, zxC;
	Mat_<double> m(3, 3);

	xs = u[1] * v[2] - u[2] * v[1];
	ys = u[2] * v[0] - u[0] * v[2];
	zs = u[0] * v[1] - u[1] * v[0];
	s = sqrt(xs * xs + ys * ys + zs * zs);
	c = u[0] * v[0] + u[1] * v[1] + u[2] * v[2];
	x = xs / s;
	y = ys / s;
	z = zs / s;
	C = 1 - c;
	xC = x * C;
	yC = y * C;
	zC = z * C;
	xyC = x * yC;
	yzC = y * zC;
	zxC = z * xC;

	m(0, 0) = x * xC + c;
	m(0, 1) = xyC - zs;
	m(0, 2) = zxC + ys;
	m(1, 0) = xyC + zs;
	m(1, 1) = y * yC + c;
	m(1, 2) = yzC - xs;
	m(2, 0) = zxC - ys;
	m(2, 1) = yzC + xs;
	m(2, 2) = z * zC + c;

	return m;
}
